import { writable, type Writable, derived, type Readable } from 'svelte/store';

export const devices: Writable<MediaDeviceInfo[]> = writable([]);


export const availableDevices: Readable<{
    audioinput: boolean,
    audiooutput: boolean,
    videoinput: boolean,
}> = derived(devices, $devices => {
    let available: { [key in MediaDeviceKind]: boolean } = {
        audioinput: false,
        audiooutput: false,
        videoinput: false,
    };

    for (const device of $devices) {
        if (available.hasOwnProperty(device.kind)) available[device.kind] = true;
    }

    return available;
})


export async function initialize() {
    await update();

    addEventListener("devicechange", event => {
        console.log(event);
        update();
    });
}

export async function update() {
    devices.set(await navigator.mediaDevices.enumerateDevices());
}

